"""
$RCSfile: TransformerRegistry.py,v $

XMLTransform automatically transforms an XML document via XSLT, where
the XML document is obtained from another Zope object via acquisition.

TransformerRegistry is a folder-like object that contains a
logically related group of XSLT transformers (aka stylesheets).  The
XSLT transformers may be any non-folder Zope object that yields a
valid XML document that is an XSLT stylesheet (application/xsl mime
type)

Each instance of XMLTransform specifies a transformer name, and the
transformer is obtained from the TransformerRegistry via
acquisition.

Author: <a href="mailto:cstrong@arielpartners.com">Craeg Strong</a>
Release: 1.0
"""

__cvstag__  = '$Name:  $'[6:-2]
__date__    = '$Date: 2002/08/26 01:02:14 $'[6:-2]
__version__ = '$Revision: 1.13 $'[10:-2]

#
# Add our base classes.  Derived classes will get this stuff automatically
#
from OFS.Folder import Folder

# peer classes
from ITransformerRegistry import ITransformerRegistry

# Zope builtins
from Globals import Acquisition, DTMLFile, MessageDialog, HTML
from Acquisition import aq_parent
from AccessControl import ClassSecurityInfo
import OFS      # we grab his _property settings
import Globals  # InitializeClass (security stuff)

# python builtins
import string

################################################################
# Defaults
################################################################

# The list of possible meta_types for transformers.
# This is the default, and can be edited at runtime via the
# transformer_meta_types property
default_transformer_meta_types = [
    "CVS File",
    "External File",
    "DTML Method",
    "DTML Document",
    "File",
    "Page Template",
    "XML Document",
    "XML File",
    "XML Transform",    
    ]

# The list of object types that could never possibly be
# transformers under any stretch of the imagination :-)
non_transformer_meta_types = [ 
    "Accelerated HTTP Cache Manager",
    "Browser Id Manager",
    "CVS Sandbox Registry",
    "External Method",
    "Folder",
    "Image",
    "Local File System",
    "Mail Host",
    "RAM Cache Manager",
    "Script (Python)",    
    "Session Data Manager",
    "Set Access Rule",
    "SiteRoot",
    "Slideshow",
    "Squishdot Site",
    "Temporary Folder",
    "Transient Object Container",
    "User Folder",
    "Version",
    "Virtual Host Monster",                               
    "Vocabulary",                               
    "XML Transform Cache Manager",
    "XSL Transformer Registry",                               
    "Z Search Interface",    
    "Z Gadfly Database Connection",
    "Z SQL Method",    
    "ZCatalog",
    "ZSyncer",
    "ZWiki Web",
    "Zope Tutorial",
    ]

################################################################
# Module Scoped Convenience Methods
################################################################

def addInstance(folder, id, title='', description=''):
    """
    
    This is a convenience factory method for creating an instance of
    TransformerRegistry.  It returns the object created, and may
    therefore be more convenient than the addTransformerRegistry()
    method it calls.  It is used by the unit testing programs.

    """
    folder.manage_addProduct['XMLTransform'].addTransformerRegistry(id,title,description)
    return folder[id]

################################################################
# Container Methods
################################################################

# globals() is standard python for passing the current namespace
manage_addTransformerRegistryForm = DTMLFile('dtml/create_registry', globals())

def manage_addTransformerRegistry(self,
                                  id, title='', description='',
                                  REQUEST=None, RESPONSE=None):
    """
    
    Add an TransformerRegistry to a folder.  Called from the
    create_registry.dtml GUI form in the Zope Management Interface.
    It calls addTransformerRegistry to actually do the work.
    
    """
    try:
        self.addTransformerRegistry(id, title, description)
        message = 'successfully created ' + id + ' registry object.'        
        return MessageDialog(title   = 'CREATED',
                             message = message,
                             action  = 'manage_main')
    except Exception, e:
        message = str(e)
        mesage.replace('\n','<br/>')
        return MessageDialog(title   = 'ERROR',
                             message = message,
                             action  = 'manage_main')

def addTransformerRegistry(self, id, title='', description=''):
    """
    
    Add an TransformerRegistry to a folder.  You should call this
    method directly if you are creating an instance of
    TransformerRegistry programatically.

    """
    if not id:
        raise Exception('Required fields must not be blank')

    self._setObject(id, TransformerRegistry(id, title,
                                            description))

################################################################
# TransformerRegistry class
################################################################

class TransformerRegistry(Folder):
    """
    XMLTransform TransformerRegistry class
    """

    meta_type = 'XSL Transformer Registry'  # This is the name Zope will use for the Product in
    # the "addProduct" list

    __implements__ = ITransformerRegistry

    # This tuple defines our properties
    _properties = (
        {'id':'title',                  'type':'string',             'mode':'w'},
        {'id':'description',            'type':'string',             'mode':'w'},
        {'id':'available_transformers', 'type':'selection',          'mode':'w',\
         'select_variable':'transformerNames'},
        {'id':'transformer_meta_types', 'type':'multiple selection', 'mode':'w',\
         'select_variable':'filteredMetaTypes'},
        )

    # inherit manage_options from Folder (see Folder.py)

    _security = ClassSecurityInfo()

    # set security for the object itself, e.g. if it is accessed in
    # DTML code This line is REQUIRED to allow access to XMLTransforms
    # by the public (ariel DTDs, etc.)
    _security.declareObjectPublic()

    def __init__(self, id, title, description):
        """
        Initialize a new instance of TransformerRegistry
        """
        self.id             = id
        self.title          = title
        self.description    = description
        self.__version__    = __version__

        # Transformers can only be of the following meta_types.
        # all other objects are ignored
        self.transformer_meta_types = default_transformer_meta_types

        # This is a dummy variable to allow us to use a property to
        # provide the user with a quick way to scan the list of
        # available transformers.  Yes, I am too lazy to create a
        # whole new DTML form for one measly little property...
        # :-) CKS 8/7/2002

        self.available_transformers = ""

    ################################################################
    # Methods implementing the ITransformerRegistry interface
    ################################################################

    _security.declarePublic('transformerNames')
    def transformerNames(self):
        """
        
        Returns the names of transformers registered with this
        registry as a sequence.  This class returns the names as
        relative URL paths, such as would be accepted by
        ObjectManagerItem.restrictedTraverse()
        
        """
        xforms = self.nonFolderObjects(self)
        # print map(lambda i: i.getId(), xforms)  # debug
        xformNames = []

        # recurse up the chain to create a relative path from us
        for xform in xforms:
            # Omit from menu, if object has 'omit' attribute
            if hasattr(xform, 'omit'): continue
            # Omit from menu, if meta_type not in our list
            if xform.meta_type not in self.transformer_meta_types: continue
            name = xform.getId()
            ob = aq_parent(xform)
            while ob.getId() != self.id:
                # retrace back up subfolders to create
                # a unique relative URL
                name = "/".join( (ob.getId(), name) )
                ob = aq_parent(ob)
                if ob is None:
                    # uh oh, we skipped ourselves (?)
                    raise Error
            xformNames.append(name)
        return xformNames

    _security.declarePublic('transformerMetaTypes')
    def transformerMetaTypes(self):
        """

        Returns the types of objects that are allowed to be
        transformers.  Objects contained within the registry that have
        meta_types that are not in this list will not be omitted, that
        is, their names will not be returned via transformerNames()

        """
        return self.transformer_meta_types
    
    _security.declarePublic('transformerObject')
    def transformerObject(self, transformerName):
        """

        Obtain the Zope object holding the XSLT, or None if the name
        does not point to a valid object.

        """

        return self.restrictedTraverse(transformerName, None)

    ################################################################
    # Utilities
    ################################################################

    def filteredMetaTypes(self):
        "Return all meta types that can possibly be transformers"
        retnames = []
        alltypes = self.all_meta_types()
        for t in alltypes:
            if t.has_key('name'):
                name = t['name']
                if name not in non_transformer_meta_types:
                    retnames.append(t['name'])
        return retnames

    def nonFolderObjects(self, folder):
        """
        return all non folder objects within a folder, recursively
        """
        objects = []
        vals    = folder.objectValues()
        for ob in vals:
            if ob.isPrincipiaFolderish:
                objects = objects + self.nonFolderObjects(ob)
            else:
                objects.append(ob)
        return objects

    ################################################################
    # Standard Zope stuff
    ################################################################

    _security.declareProtected('View', '__str__')
    def __str__(self, REQUEST = None):
        "return contents as string"
        message = self.id + " (" + self.title + ")"
        return message

    ################################################################
    # Support for Schema Migration
    ################################################################

    def repair(self):
        """

        Repair this object.  This method is used for schema migration,
        when the class definition changes and existing instances of
        previous versions must be updated.

        """
        #if not hasattr(self,'_caching'):
        #    self._caching        = 'on'

# register security information
Globals.InitializeClass(TransformerRegistry)
